/*
 * Copyright (c) 2005, Innoopract Informationssysteme GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *  Innoopract - initial API and implementation
 */
package bug2html;

import java.util.Comparator;

/**
 * Comparator for Bug instances.
 * @author Elias Volanakis <evolanakis@innoopract.de>
 */
public class BugComparator implements Comparator {

  /**
   * Orders bugs by Milestone, Priority, Component and Id.
   * @param o1 a non-null Bug instance
   * @param o2 a non-null Bug instance
   * @return a negative integer, zero or positive integer, 
   *         if the o1 is less than, equal, or greater than o2  
   * @throws NullPointerException if any parameter is null
   * @throws ClassCastException if any parameter is not a Bug instance
   */
  public int compare(final Object o1, final Object o2) {
    Bug bug1 = (Bug) o1;
    Bug bug2 = (Bug) o2;

    int result = bug1.getMilestone().compareTo(bug2.getMilestone());
    if (result == 0) {
      result = bug1.getPriority().compareTo(bug2.getPriority());
    }
    if (result == 0) {
      result = bug1.getComponent().compareTo(bug2.getComponent());
    }
    if (result == 0) {
      result = bug1.getId() - bug2.getId(); // valid Ids start at 1
    }
    return result;
  }
}
